@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,7 @@
 
 
 #include <kpmainwindow.h>
+#include <kpmainwindow_p.h>
 
 #include <qdragobject.h>
 #include <qpainter.h>
@@ -51,10 +52,11 @@
 #include <kpthumbnail.h>
 #include <kptool.h>
 #include <kptooltoolbar.h>
-#include <kpview.h>
 #include <kpviewmanager.h>
 #include <kpviewscrollablecontainer.h>
 #include <kpwidgetmapper.h>
+#include <kpzoomedthumbnailview.h>
+#include <kpzoomedview.h>
 
 #if DEBUG_KP_MAIN_WINDOW
     #include <qdatetime.h>
@@ -123,12 +125,17 @@ void kpMainWindow::readGeneralSettings ()
     m_configShowGrid = cfg->readBoolEntry (kpSettingShowGrid, false);
     m_configShowPath = cfg->readBoolEntry (kpSettingShowPath, false);
     m_configColorSimilarity = cfg->readDoubleNumEntry (kpSettingColorSimilarity, 0);
+    d->m_moreEffectsDialogLastEffect = cfg->readNumEntry (kpSettingMoreEffectsLastEffect);
+    d->m_resizeScaleDialogLastKeepAspect = cfg->readBoolEntry (kpSettingResizeScaleLastKeepAspect, false);
+
 
 #if DEBUG_KP_MAIN_WINDOW
     kdDebug () << "\t\tGeneral Settings: firstTime=" << m_configFirstTime
                << " showGrid=" << m_configShowGrid
                << " showPath=" << m_configShowPath
                << " colorSimilarity=" << m_configColorSimilarity
+               << " moreEffectsDialogLastEffect=" << d->m_moreEffectsDialogLastEffect
+               << " resizeScaleDialogLastKeepAspect=" << d->m_resizeScaleDialogLastKeepAspect
                << endl;
 #endif
 }
@@ -145,10 +152,14 @@ void kpMainWindow::readThumbnailSettings ()
 
     m_configThumbnailShown = cfg->readBoolEntry (kpSettingThumbnailShown, false);
     m_configThumbnailGeometry = cfg->readRectEntry (kpSettingThumbnailGeometry);
+    m_configZoomedThumbnail = cfg->readBoolEntry (kpSettingThumbnailZoomed, true);
+    d->m_configThumbnailShowRectangle = cfg->readBoolEntry (kpSettingThumbnailShowRectangle, true);
 
 #if DEBUG_KP_MAIN_WINDOW
     kdDebug () << "\t\tThumbnail Settings: shown=" << m_configThumbnailShown
                << " geometry=" << m_configThumbnailGeometry
+               << " zoomed=" << m_configZoomedThumbnail
+               << " showRectangle=" << d->m_configThumbnailShowRectangle
                << endl;
 #endif
 }
@@ -352,6 +363,8 @@ void kpMainWindow::setupActions ()
     setupViewMenuActions ();
     setupImageMenuActions ();
     setupSettingsMenuActions ();
+    setupHelpMenuActions ();
+
     setupTextToolBarActions ();
     setupToolActions ();
 }
@@ -364,6 +377,7 @@ void kpMainWindow::enableDocumentActions (bool enable)
     enableViewMenuDocumentActions (enable);
     enableImageMenuDocumentActions (enable);
     enableSettingsMenuDocumentActions (enable);
+    enableHelpMenuDocumentActions (enable);
 }
 
 
@@ -511,8 +525,10 @@ void kpMainWindow::setDocument (kpDocument *newDoc)
     #if DEBUG_KP_MAIN_WINDOW
         kdDebug () << "\tcreating views" << endl;
     #endif
-        m_mainView = new kpView (m_scrollView->viewport (), "mainView", this,
-                                 m_document->width (), m_document->height ());
+        m_mainView = new kpZoomedView (m_document, m_toolToolBar, m_viewManager,
+                                       0/*buddyView*/,
+                                       m_scrollView,
+                                       m_scrollView->viewport (), "mainView");
         if (m_scrollView)
         {
             m_scrollView->addChild (m_mainView);
@@ -591,7 +607,7 @@ void kpMainWindow::setDocument (kpDocument *newDoc)
         connect (m_document, SIGNAL (contentsChanged (const QRect &)),
                  m_viewManager, SLOT (updateViews (const QRect &)));
         connect (m_document, SIGNAL (sizeChanged (int, int)),
-                 m_viewManager, SLOT (resizeViews (int, int)));
+                 m_viewManager, SLOT (adjustViewsToEnvironment ()));
 
     #if DEBUG_KP_MAIN_WINDOW
         kdDebug () << "\tenabling actions" << endl;
@@ -783,13 +799,19 @@ void kpMainWindow::dropEvent (QDropEvent *e)
         if (view)
         {
             const QPoint viewPos = view->mapFromGlobal (globalPos);
+            const QPoint docPoint = view->transformViewToDoc (viewPos);
 
-            selTopLeft = view->zoomViewToDoc (viewPos);
+            // viewUnderCursor() is hacky and can return a view when we aren't
+            // over one thanks to drags.
+            if (m_document && m_document->rect ().contains (docPoint))
+            {
+                selTopLeft = docPoint;
 
-            // TODO: In terms of doc pixels, would be inconsistent behaviour
-            //       based on zoomLevel of view.
-            // selTopLeft -= QPoint (-view->selectionResizeHandleAtomicSize (),
-            //                       -view->selectionResizeHandleAtomicSize ());
+                // TODO: In terms of doc pixels, would be inconsistent behaviour
+                //       based on zoomLevel of view.
+                // selTopLeft -= QPoint (-view->selectionResizeHandleAtomicSize (),
+                //                       -view->selectionResizeHandleAtomicSize ());
+            }
         }
 
         pasteText (text, true/*force new text selection*/, selTopLeft);
@@ -838,48 +860,6 @@ void kpMainWindow::moveEvent (QMoveEvent * /*e*/)
 }
 
 
-// public
-void kpMainWindow::drawTransparentBackground (QPainter *painter,
-                                              int /*viewWidth*/, int /*viewHeight*/,
-                                              const QRect &rect,
-                                              bool isPreview)
-{
-    const int cellSize = !isPreview ? 16 : 10;
-
-    int starty = rect.y ();
-    if (starty % cellSize)
-        starty -= (starty % cellSize);
-
-    int startx = rect.x ();
-    if (startx % cellSize)
-        startx -= (startx % cellSize);
-
-    painter->save ();
-    for (int y = starty; y <= rect.bottom (); y += cellSize)
-    {
-        for (int x = startx; x <= rect.right (); x += cellSize)
-        {
-            bool parity = (x / cellSize + y / cellSize) % 2;
-            QColor col;
-
-            if (parity)
-            {
-                if (!isPreview)
-                    col = QColor (213, 213, 213);
-                else
-                    col = QColor (224, 224, 224);
-            }
-            else
-                col = Qt::white;
-
-            painter->fillRect (x - rect.x (), y - rect.y (), cellSize, cellSize,
-                               col);
-        }
-    }
-    painter->restore ();
-}
-
-
 // private slot
 void kpMainWindow::slotUpdateCaption ()
 {
@@ -905,3 +885,4 @@ void kpMainWindow::slotDocumentRestored ()
 
 
 #include <kpmainwindow.moc>
+
