@@ -86,7 +86,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	KGlobal::locale()->insertCatalogue("kpdf");
 
 	// create browser extension (for printing when embedded into browser)
-	new BrowserExtension(this);
+	m_bExtension = new BrowserExtension(this);
 
 	// xpdf 'extern' global class (m_count is a static instance counter)
 	//if ( m_count ) TODO check if we need to insert these lines..
@@ -102,7 +102,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_document = new KPDFDocument();
 	connect( m_document, SIGNAL( linkFind() ), this, SLOT( slotFind() ) );
 	connect( m_document, SIGNAL( linkGoToPage() ), this, SLOT( slotGoToPage() ) );
-	connect( m_document, SIGNAL( openURL(const KURL &) ), this, SLOT( openURL(const KURL &) ) );
+	connect( m_document, SIGNAL( openURL(const KURL &) ), this, SLOT( openURLFromDocument(const KURL &) ) );
 
 	// widgets: ^searchbar (toolbar containing label and SearchWidget)
 //	m_searchToolBar = new KToolBar( parentWidget, "searchBar" );
@@ -117,7 +117,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_splitter->setOpaqueResize( true );
 	setWidget( m_splitter );
 	m_watchFile = new KToggleAction( i18n( "&Watch File" ), 0, this, SLOT( slotWatchFile() ), actionCollection(), "watch_file" );
-	m_watchFile->setChecked( Settings::watchFile() );
+	m_watchFile->setChecked( KpdfSettings::watchFile() );
 
 	// widgets: [left panel] | []
 	m_leftPanel = new QWidget( m_splitter );
@@ -140,7 +140,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_searchWidget = new SearchWidget( thumbsBox, m_document );
 	m_thumbnailList = new ThumbnailList( thumbsBox, m_document );
 //	ThumbnailController * m_tc = new ThumbnailController( thumbsBox, m_thumbnailList );
-	connect( m_thumbnailList, SIGNAL( urlDropped( const KURL& ) ), SLOT( openURL( const KURL & )) );
+	connect( m_thumbnailList, SIGNAL( urlDropped( const KURL& ) ), SLOT( openURLFromDocument( const KURL & )) );
 	connect( m_thumbnailList, SIGNAL( rightClick(const KPDFPage *, const QPoint &) ), this, SLOT( slotShowMenu(const KPDFPage *, const QPoint &) ) );
 	// shrink the bottom controller toolbar (too hackish..)
 	thumbsBox->setStretchFactor( m_searchWidget, 100 );
@@ -169,7 +169,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	// widgets: [] | [right 'pageView']
 	m_pageView = new PageView( m_splitter, m_document );
 	m_pageView->setFocus(); //usability setting
-	connect( m_pageView, SIGNAL( urlDropped( const KURL& ) ), SLOT( openURL( const KURL & )));
+	connect( m_pageView, SIGNAL( urlDropped( const KURL& ) ), SLOT( openURLFromDocument( const KURL & )));
 	connect( m_pageView, SIGNAL( rightClick(const KPDFPage *, const QPoint &) ), this, SLOT( slotShowMenu(const KPDFPage *, const QPoint &) ) );
 
 	// add document observers
@@ -236,7 +236,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	m_pageView->setupActions( ac );
 
 	// apply configuration (both internal settings and GUI configured items)
-	QValueList<int> splitterSizes = Settings::splitterSizes();
+	QValueList<int> splitterSizes = KpdfSettings::splitterSizes();
 	if ( !splitterSizes.count() )
 	{
 		// the first time use 1/10 for the panel and 9/10 for the pageView
@@ -244,6 +244,9 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 		splitterSizes.push_back( 500 );
 	}
 	m_splitter->setSizes( splitterSizes );
+	// get notified about splitter size changes (HACK that will be removed
+	// by connecting to Qt4::QSplitter's sliderMoved())
+	m_pageView->installEventFilter( this );
 	slotNewConfig();
 
 	m_watcher = new KDirWatch( this );
@@ -253,7 +256,8 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
 	// [SPEECH] check for KTTSD presence and usability
 	KTrader::OfferList offers = KTrader::self()->query("DCOP/Text-to-Speech", "Name == 'KTTSD'");
-	Settings::setUseKTTSD( (offers.count() > 0) );
+	KpdfSettings::setUseKTTSD( (offers.count() > 0) );
+	KpdfSettings::writeConfig();
 
 	// set our XML-UI resource file
 	setXMLFile("part.rc");
@@ -263,11 +267,6 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
 Part::~Part()
 {
-    // save internal settings
-    Settings::setSplitterSizes( m_splitter->sizes() );
-    // write to disk config file
-    Settings::writeConfig();
-
     delete m_document;
     if ( --m_count == 0 )
         delete globalParams;
@@ -314,7 +313,16 @@ KAboutData* Part::createAboutData()
 
 bool Part::openFile()
 {
-    bool ok = m_document->openDocument( m_file, url() );
+    KMimeType::Ptr mime;
+    if ( m_bExtension->urlArgs().serviceType.isEmpty() )
+    {
+        mime = KMimeType::findByPath( m_file );
+    }
+    else
+    {
+        mime = KMimeType::mimeType( m_bExtension->urlArgs().serviceType );
+    }
+    bool ok = m_document->openDocument( m_file, url(), mime );
 
     // update one-time actions
     m_find->setEnabled( ok );
@@ -346,6 +354,13 @@ bool Part::openFile()
     return true;
 }
 
+void Part::openURLFromDocument(const KURL &url)
+{
+    m_bExtension->openURLNotify();
+    m_bExtension->setLocationBarURL(url.prettyURL());
+    openURL(url);
+}
+
 bool Part::openURL(const KURL &url)
 {
     // note: this can be the right place to check the file for gz or bz2 extension
@@ -356,6 +371,8 @@ bool Part::openURL(const KURL &url)
     bool b = KParts::ReadOnlyPart::openURL(url);
     if ( !b )
         KMessageBox::error( widget(), i18n("Could not open %1").arg( url.prettyURL() ) );
+    else
+        m_viewportDirty.pageNumber = -1;
     return b;
 }
 
@@ -375,9 +392,19 @@ bool Part::closeURL()
     return KParts::ReadOnlyPart::closeURL();
 }
 
+bool Part::eventFilter( QObject * watched, QEvent * e )
+{
+    // if pageView has been resized, save splitter sizes
+    if ( watched == m_pageView && e->type() == QEvent::Resize )
+        saveSplitterSize();
+
+    // only intercept events, don't block them
+    return false;
+}
+
 void Part::slotWatchFile()
 {
-  Settings::setWatchFile(m_watchFile->isChecked());
+  KpdfSettings::setWatchFile(m_watchFile->isChecked());
   if( m_watchFile->isChecked() )
     m_watcher->startScan();
   else
@@ -402,11 +429,22 @@ void Part::slotFileDirty( const QString& fileName )
 
 void Part::slotDoFileDirty()
 {
-  uint p = m_document->currentPage() + 1;
-  if (openFile())
+  if (m_viewportDirty.pageNumber == -1)
   {
-    if (p > m_document->pages()) p = m_document->pages();
-    goToPage(p);
+    m_viewportDirty = m_document->viewport();
+    m_pageView->showText(i18n("Reloading the document..."), 0);
+  }
+
+  if (KParts::ReadOnlyPart::openURL(m_file))
+  {
+    if (m_viewportDirty.pageNumber >= (int)m_document->pages()) m_viewportDirty.pageNumber = (int)m_document->pages() - 1;
+    m_document->setViewport(m_viewportDirty);
+    m_viewportDirty.pageNumber = -1;
+  }
+  else
+  {
+    m_watcher->addFile(m_file);
+    m_dirtyHandler->start( 750, true );
   }
 }
 
@@ -442,6 +480,12 @@ void Part::enableTOC(bool enable)
 	m_toolBox->setItemEnabled(0, enable);
 }
 
+void Part::saveSplitterSize()
+{
+    KpdfSettings::setSplitterSizes( m_splitter->sizes() );
+    KpdfSettings::writeConfig();
+} 
+
 //BEGIN go to page dialog
 class KPDFGotoPageDialog : public KDialogBase
 {
@@ -542,6 +586,12 @@ void Part::slotSaveFileAs()
     KURL saveURL = KFileDialog::getSaveURL( url().isLocalFile() ? url().url() : url().fileName(), QString::null, widget() );
     if ( saveURL.isValid() && !saveURL.isEmpty() )
     {
+        if ( KIO::NetAccess::exists( saveURL, false, widget() ) )
+        {
+            if (KMessageBox::questionYesNo( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?").arg(saveURL.filename())) != KMessageBox::Yes)
+                return;
+        }
+
         if ( !KIO::NetAccess::file_copy( url(), saveURL, -1, true ) )
             KMessageBox::information( 0, i18n("File could not be saved in '%1'. Try to save it to another location.").arg( saveURL.prettyURL() ) );
     }
@@ -555,7 +605,7 @@ void Part::slotPreferences()
         return;
 
     // we didn't find an instance of this dialog, so lets create it
-    PreferencesDialog * dialog = new PreferencesDialog( m_pageView, Settings::self() );
+    PreferencesDialog * dialog = new PreferencesDialog( m_pageView, KpdfSettings::self() );
     // keep us informed when the user changes settings
     connect( dialog, SIGNAL( settingsChanged() ), this, SLOT( slotNewConfig() ) );
 
@@ -568,7 +618,7 @@ void Part::slotNewConfig()
     // changed before applying changes.
 
     // Left Panel and search Widget
-    bool showLeft = Settings::showLeftPanel();
+    bool showLeft = KpdfSettings::showLeftPanel();
     if ( m_leftPanel->isShown() != showLeft )
     {
         // show/hide left qtoolbox
@@ -576,12 +626,12 @@ void Part::slotNewConfig()
         // this needs to be hidden explicitly to disable thumbnails gen
         m_thumbnailList->setShown( showLeft );
     }
-    bool showSearch = Settings::showSearchBar();
+    bool showSearch = KpdfSettings::showSearchBar();
     if ( m_searchWidget->isShown() != showSearch )
         m_searchWidget->setShown( showSearch );
 
     // Main View (pageView)
-    QScrollView::ScrollBarMode scrollBarMode = Settings::showScrollBars() ?
+    QScrollView::ScrollBarMode scrollBarMode = KpdfSettings::showScrollBars() ?
         QScrollView::AlwaysOn : QScrollView::AlwaysOff;
     if ( m_pageView->hScrollBarMode() != scrollBarMode )
     {
@@ -593,7 +643,7 @@ void Part::slotNewConfig()
     m_document->reparseConfig();
 
     // update Main View and ThumbnailList contents
-    // TODO do this only when changing Settings::renderMode()
+    // TODO do this only when changing KpdfSettings::renderMode()
     m_pageView->updateContents();
     if ( showLeft && m_thumbnailList->isShown() )
         m_thumbnailList->updateWidgets();
