@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2001 Wilco Greven <greven@kde.org>
+   Copyright (C) 2004-2005 Wilfried Huss <Wilfried.Huss@gmx.at>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -79,9 +80,12 @@ public slots:
     /** Turn the scrollbars on/off. */
     void slotShowScrollbars(bool);
 
-    /** If the viewport is larger than the page, center the page on
-        the viewport. */
-    void centerContents();
+    /** Set layout of the page widgets according to the current viewmode and zoomlevel.
+        Set zoomChanged = true if the the layout needs updateing because the zoomlevel has changed. */
+    void centerContents(bool zoomChanged = false);
+
+    /** Make the 'y' coordinate of the specified widget visible */
+    void slotShowWidget(QWidget *wdg, Q_UINT16 y);
 
 signals:
     void viewSizeChanged( QSize size );
@@ -92,16 +96,12 @@ signals:
     void wheelEventReceived( QWheelEvent * );
 
 protected:
-    /** This method draws shadows about the child widgets that the
-        scrollview holds. It is automatically called by QT whenever
-        necessary. */
-    void drawContents ( QPainter * p, int clipx, int clipy, int clipw, int cliph );
-
     void keyPressEvent( QKeyEvent* );
 
     /** Reimplemented from QScrollView to make sure that the page is
         centered when it fits in the viewport. */
     void viewportResizeEvent( QResizeEvent* );
+    void viewportPaintEvent(QPaintEvent*);
 
     /** Reimplemented from QScrollView, because the kviepart needs to
         handle the wheel events itself. The wheel event is passed on by 
@@ -127,20 +127,8 @@ private:
     bool continuousViewmode;
     bool fullScreen;
 
-    /** Color used internally in the shadow drawing routines in the
-        drawContents method. */
-    QColor   viewportBackgroundColorForCorners;
-
-    /** Internal storages used in the shadow drawing routines in the
-        drawContents method.*/
-    QPixmap  URShadow, BRShadow, BLShadow;
-
     QPtrVector<QWidget> widgetListIfOldAddChildIsUsed;
 
-    /** This is set in slotShowScrollbars() to remember the status of the
-        scrollbars when we switch from fullscreen mode back to normal mode. */
-    bool showScrollBars;
-
     /** This int remembers the style of the frame of the centering
         scrollview when fullscreen mode is switched on. It is then
         restored when it is switched off. */
