@@ -701,9 +701,9 @@ void KMultiPage::repaintAllVisibleWidgets(void)
 
     // Resize, if necessary
     QSize pageSize = pageCache.sizeOfPageInPixel(i+1);
-    if (pageSize != _documentWidget->size()) 
+    if (pageSize != _documentWidget->pageSize()) 
     {
-      _documentWidget->resize(pageSize);
+      _documentWidget->setPageSize(pageSize);
       everResized = true;
     }
   }
@@ -713,7 +713,7 @@ void KMultiPage::repaintAllVisibleWidgets(void)
   // widgets. If no widgets were resized, go through the list of
   // widgets again, and update those that are visible.
   if (everResized == true)
-    scrollView()->centerContents();
+    scrollView()->centerContents(true);
   else 
   {
     QRect visiblRect(scrollView()->contentsX(), scrollView()->contentsY(),
@@ -775,7 +775,10 @@ void KMultiPage::setNumberOfPages(int numberOfPages)
 
 Q_UINT16 KMultiPage::numberOfPages()
 {
-  return markList()->numberOfPages();
+  if (renderer == 0)
+    return 0;
+
+  return renderer->totalPages();
 }
 
 QValueList<int> KMultiPage::selectedPages()
@@ -869,9 +872,11 @@ void KMultiPage::prevPage()
   Q_UINT8 cols = scrollView()->getNrColumns();
   Q_UINT8 rows = scrollView()->getNrRows();
 
-  Q_UINT16 np = 0;
+  Q_UINT16 np = 1;
   if (cols*rows < currentPageNumber())
+  {
     np = currentPageNumber() - cols*rows;
+  }
 
   gotoPage(np);
 }
@@ -910,15 +915,15 @@ void KMultiPage::scroll(Q_INT32 deltaInPixel)
 
   if (deltaInPixel < 0) {
     if (scrollBar->value() == scrollBar->minValue()) {
-      if ( (currentPageNumber() == 0) || (changePageDelayTimer.isActive()) )
+      if ( (currentPageNumber() == 1) || (changePageDelayTimer.isActive()) )
         return;
 
       if (scrollView()->isContinuous())
         return;
 
       changePageDelayTimer.stop();
-
       prevPage();
+
       scrollView()->setContentsPos(scrollView()->contentsX(), scrollBar->maxValue());
       return;
     }
@@ -926,7 +931,7 @@ void KMultiPage::scroll(Q_INT32 deltaInPixel)
 
   if (deltaInPixel > 0) {
     if (scrollBar->value() == scrollBar->maxValue()) {
-      if ( (currentPageNumber() + 1 == numberOfPages()) || (changePageDelayTimer.isActive()) )
+      if ( (currentPageNumber() == numberOfPages()) || (changePageDelayTimer.isActive()) )
         return;
 
       if (scrollView()->isContinuous())
@@ -1025,7 +1030,7 @@ void KMultiPage::readDown()
     if (sv->isContinuous())
       return;
 
-    if (currentPageNumber() + 1 == numberOfPages())
+    if (currentPageNumber() == numberOfPages())
       return;
 
     nextPage();
@@ -1045,7 +1050,7 @@ void KMultiPage::readUp()
     if (sv->isContinuous())
       return;
 
-    if (currentPageNumber() == 0)
+    if (currentPageNumber() == 1)
       return;
 
     prevPage();
@@ -1067,8 +1072,6 @@ void KMultiPage::jumpToReference(QString reference)
 
 void KMultiPage::gotoPage(const anchor &a)
 {
-  kdError() << "GOTOPAGE pg=" << a.page << ", dist=" << a.distance_from_top_in_inch << endl;
-
   if (a.page.isInvalid() || (renderer == 0))
     return;
 
@@ -1126,7 +1129,6 @@ void KMultiPage::doSelectAll(void)
 }
 
 
-
 void  KMultiPage::showFindTextDialog(void)
 {
   if ((renderer == 0) || (renderer->supportsTextSearch() == false))
@@ -1430,3 +1432,14 @@ void KMultiPage::findPrevText(void)
     }
   } // of while (all pages)
 }
+
+
+void KMultiPage::enableActions(bool fileLoaded)
+{
+  bool textSearch = false;
+  if (fileLoaded && (getRenderer() != 0) && getRenderer()->supportsTextSearch())
+    textSearch = true;
+
+  findTextAction->setEnabled(textSearch);
+  selectAllAction->setEnabled(textSearch);
+}
